/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Community System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.impl;

import java.sql.*;
import java.util.*;
import org.apache.log4j.*;
import com.silverwrist.venice.db.*;
import com.silverwrist.venice.core.*;

class CategoryDescriptorImpl implements CategoryDescriptor, Cloneable
{
  /*--------------------------------------------------------------------------------
   * Internal class for saving category segments
   *--------------------------------------------------------------------------------
   */

  static class CatSegment
  {
    private int id;
    private String name;

    CatSegment(int id, String name)
    {
      this.id = id;
      this.name = name;

    } // end constructor

    public int getID()
    {
      return id;

    } // end getID

    public String getName()
    {
      return name;

    } // end getName

  } // end class CatSegment

  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(CategoryDescriptorImpl.class.getName());

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private DataPool datapool;       // used for doing database lookups
  private Vector cats;             // the actual category segments
  private int symlink = -1;        // if our category is actually a symlink
  private boolean do_hide = true;  // do we hide subcategories marked hide_dir?

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  CategoryDescriptorImpl(DataPool datapool, int catid, boolean do_hide) throws DataException
  {
    this.datapool = datapool;
    cats = new Vector();
    this.do_hide = do_hide;

    if (catid<0)
      return;  // this is a "top" category

    Connection conn = null;

    try
    { // get a connection and a prepared statement
      conn = datapool.getConnection();
      doFillFromTop(conn,catid);

    } // end try
    catch (SQLException e)
    { // remap to DataException
      throw new DataException("Unable to load category descriptor: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure and release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end constructor

  protected CategoryDescriptorImpl(DataPool datapool, Connection conn, int catid, boolean do_hide)
    throws SQLException, DataException
  {
    this.datapool = datapool;
    cats = new Vector();
    this.do_hide = do_hide;

    if (catid<0)
      return;  // this is a "top" category

    doFillFromTop(conn,catid);

  } // end constructor

  protected CategoryDescriptorImpl(DataPool datapool, int id, int symlink, String name, boolean do_hide)
  {
    this.datapool = datapool;
    this.cats = new Vector();
    this.symlink = symlink;
    this.do_hide = do_hide;

    this.cats.add(new CatSegment(id,name));
    this.cats.trimToSize();

  } // end constructor

  protected CategoryDescriptorImpl(CategoryDescriptorImpl other, int copy_levels)
  {
    this.datapool = other.datapool;
    this.cats = new Vector();
    this.symlink = ((copy_levels==other.cats.size()) ? other.symlink : -1);
    this.do_hide = other.do_hide;

    if (copy_levels>0)
    { // copy the references to the objects directly
      for (int i=0; i<copy_levels; i++)
	this.cats.add(other.cats.get(i));
      this.cats.trimToSize();

    } // end if

  } // end constructor

  protected CategoryDescriptorImpl(CategoryDescriptorImpl other, int id, int symlink, String name)
  {
    this.datapool = other.datapool;
    this.cats = new Vector();
    this.symlink = symlink;
    this.do_hide = other.do_hide;

    // copy the references to the objects directly
    for (int i=0; i<other.cats.size(); i++)
      this.cats.add(other.cats.get(i));
    this.cats.add(new CatSegment(id,name));
    this.cats.trimToSize();

  } // end constructor

  /*--------------------------------------------------------------------------------
   * finalize() function
   *--------------------------------------------------------------------------------
   */

  protected void finalize()
  {
    datapool = null;
    cats = null;

  } // end finalize

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private void doFillFromTop(Connection conn, int catid) throws SQLException, DataException
  {
    PreparedStatement stmt = conn.prepareStatement("SELECT parent, symlink, name "
						   + "FROM refcategory WHERE catid = ?;");
    int curr_catid = catid;
    while (curr_catid!=-1)
    { // get the category reference for this level
      stmt.setInt(1,curr_catid);
      ResultSet rs = stmt.executeQuery();

      if (!(rs.next()))  // unable to read database
	throw new DataException("category #" + String.valueOf(catid) + " could not be resolved");
	
      // Save off the symbolic link value, if this category is a symlink.
      int symlink_field = rs.getInt("symlink");
      if (symlink_field!=-1)
      { // handle saving our symlink value - can only occur at END of path
	if (cats.size()>0)
	  throw new InternalStateError("invalid symlink in category table (" + String.valueOf(curr_catid)
				       + " => " + String.valueOf(symlink_field) + ")");
	symlink = symlink_field;

      } // end if

      cats.add(0,new CatSegment(curr_catid,rs.getString("name")));
      curr_catid = rs.getInt("parent");

    } // end while

    cats.trimToSize();  // shrink vector down to size

  } // end doFillFromTop

  /*--------------------------------------------------------------------------------
   * Implementations from interface CategoryDescriptor
   *--------------------------------------------------------------------------------
   */

  public int getCategoryID()
  {
    if (cats.size()>0)
      return ((CatSegment)(cats.lastElement())).getID();
    else
      return -1;

  } // end getCategoryID

  public int getNumLevels()
  {
    return cats.size();

  } // end getNumLevels

  public int getIDAtLevel(int level)
  {
    return ((CatSegment)(cats.get(level))).getID();

  } // end getIDAtLevel

  public String getTitleAtLevel(int level)
  {
    return ((CatSegment)(cats.get(level))).getName();

  } // end getTitleAtLevel

  public List getSubCategories() throws DataException
  {
    if (symlink!=-1)
    { // "snap" the symlink before getting subcategories
      CategoryDescriptorImpl real_obj = new CategoryDescriptorImpl(datapool,symlink,do_hide);
      return real_obj.getSubCategories();

    } // end if

    Connection conn = null;
    Vector rc = new Vector();
    try
    { // get a connection and create a statement
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT catid, symlink, name FROM refcategory WHERE parent = ");
      sql.append(getCategoryID());
      if (do_hide)
	sql.append(" AND hide_dir = 0");
      sql.append(';');
      
      // run the database query
      ResultSet rs = stmt.executeQuery(sql.toString());
      while (rs.next())
      { // turn data values into CategoryDescriptor objects
	CategoryDescriptor ncd = new CategoryDescriptorImpl(this,rs.getInt("catid"),rs.getInt("symlink"),
							    rs.getString("name"));
	rc.add(ncd);

      } // end while

    } // end try
    catch (SQLException e)
    { // remap to DataException
      throw new DataException("Unable to load subcategories: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure and release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

    // wrap the vector in a ReadOnlyVector object
    return new ReadOnlyVector(rc);

  } // end getSubCategories

  public CategoryDescriptor getSuperCategory(int levels)
  {
    if ((levels<0) || (levels>(cats.size()-1)))
      throw new IndexOutOfBoundsException("invalid supercategory level value");
    return new CategoryDescriptorImpl(this,levels);

  } // end getSuperCategory

  public CategoryDescriptor getParentCategory()
  {
    return getSuperCategory(cats.size()-1);

  } // end getParentCategory

  public int getLinkedCategoryID()
  {
    if (symlink!=-1)
      return symlink;
    else
      return getCategoryID();

  } // end getLinkedCategoryID

  public boolean isSymbolicLink()
  {
    return (symlink!=-1);

  } // end isSymbolicLink

  public CategoryDescriptor getLinkedCategory() throws DataException
  {
    if (symlink!=-1)
      return new CategoryDescriptorImpl(datapool,symlink,do_hide);
    else
      return this;

  } // end getLinkedCategory

  public boolean equals(Object obj)
  {
    if (obj==null)
      return false;
    if (obj instanceof CategoryDescriptor)
    { // this is something we can compare to directly
      CategoryDescriptor other = (CategoryDescriptor)obj;
      return (other.getLinkedCategoryID()==getLinkedCategoryID());

    } // end if

    // compare string values
    return obj.toString().equals(toString());

  } // end equals

  public int hashCode()
  {
    if (symlink!=-1)
      return symlink;
    else
      return getCategoryID();
      
  } // end hashCode

  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    Iterator it = cats.iterator();
    while (it.hasNext())
    { // concatenate all the parts together
      CatSegment seg = (CatSegment)(it.next());
      if (buf.length()>0)
	buf.append(": ");
      buf.append(seg.getName());

    } // end while

    return buf.toString();

  } // end toString

  public Object clone()
  {
    return new CategoryDescriptorImpl(this,cats.size());

  } // end clone

  /*--------------------------------------------------------------------------------
   * Static operations for use within implementation package
   *--------------------------------------------------------------------------------
   */

  static List getTopLevelCategoryList(DataPool datapool, boolean do_hide) throws DataException
  {
    Connection conn = null;
    Vector rc = new Vector();
    try
    { // get a connection and create a statement
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT catid, symlink, name FROM refcategory WHERE parent = -1");
      if (do_hide)
	sql.append(" AND hide_dir = 0");
      sql.append(';');
      
      // run the database query
      ResultSet rs = stmt.executeQuery(sql.toString());
      while (rs.next())
      { // turn data values into CategoryDescriptor objects
	CategoryDescriptor ncd = new CategoryDescriptorImpl(datapool,rs.getInt("catid"),rs.getInt("symlink"),
							    rs.getString("name"),do_hide);
	rc.add(ncd);

      } // end while

    } // end try
    catch (SQLException e)
    { // remap to DataException
      throw new DataException("Unable to load subcategories: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure and release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

    // wrap the vector in a ReadOnlyVector object
    return new ReadOnlyVector(rc);

  } // end getTopLevelCategoryList

  static List searchForCategories(DataPool datapool, boolean do_hide, boolean search_all, int mode,
				  String term, int offset, int count) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("Category search: mode = " + String.valueOf(mode) + ", term '" + term + "', offset = "
		   + String.valueOf(offset) + ", count = " + String.valueOf(count));

    Vector rc = new Vector();
    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT catid FROM refcategory WHERE name ");

      switch (mode)
      { // compose SQL in different ways depending on the search term
	case SearchMode.SEARCH_PREFIX:
	  sql.append("LIKE '").append(SQLUtil.encodeStringWildcards(term)).append("%'");
	  break;

	case SearchMode.SEARCH_SUBSTRING:
	  sql.append("LIKE '%").append(SQLUtil.encodeStringWildcards(term)).append("%'");
	  break;

	case SearchMode.SEARCH_REGEXP:
	  sql.append("REGEXP '").append(SQLUtil.encodeString(term)).append('\'');
	  break;

	default:
	  throw new DataException("invalid search mode specified");

      } // end switch

      if (do_hide)
	sql.append(" AND hide_dir = 0");
      if (!search_all)
	sql.append(" AND hide_search = 0");
      sql.append(" ORDER BY parent, name LIMIT ").append(offset).append(", ").append(count+1).append(';');

      if (logger.isDebugEnabled())
	logger.debug("SQL: " + sql.toString());

      // launch the search!
      ResultSet rs = stmt.executeQuery(sql.toString());

      int[] rc_raw = new int[count+1];
      int n = 0;

      while (rs.next())  // just read off the matching CATIDs for now
	rc_raw[n++] = rs.getInt("catid");

      for (int i=0; i<n; i++)
      { // convert all the simple category IDs into full-blown CategoryDescriptor objects
	CategoryDescriptor tmp = new CategoryDescriptorImpl(datapool,conn,rc_raw[i],do_hide);
	rc.add(tmp);

      } // end for

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading SIG entries: " + e.getMessage(),e);
      throw new DataException("unable to retrieve user SIG information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

    return new ReadOnlyVector(rc);

  } // end searchForCategories

  static int getSearchCategoryCount(DataPool datapool, boolean do_hide, boolean search_all, int mode,
				    String term) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("Category search: mode = " + String.valueOf(mode) + ", term '" + term + "'");

    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT COUNT(*) AS total FROM refcategory WHERE name ");

      switch (mode)
      { // compose SQL in different ways depending on the search term
	case SearchMode.SEARCH_PREFIX:
	  sql.append("LIKE '").append(SQLUtil.encodeStringWildcards(term)).append("%'");
	  break;

	case SearchMode.SEARCH_SUBSTRING:
	  sql.append("LIKE '%").append(SQLUtil.encodeStringWildcards(term)).append("%'");
	  break;

	case SearchMode.SEARCH_REGEXP:
	  sql.append("REGEXP '").append(SQLUtil.encodeString(term)).append('\'');
	  break;

	default:
	  throw new DataException("invalid search mode specified");

      } // end switch

      if (do_hide)
	sql.append(" AND hide_dir = 0");
      if (!search_all)
	sql.append(" AND hide_search = 0");
      sql.append(';');

      if (logger.isDebugEnabled())
	logger.debug("SQL: " + sql.toString());

      // launch the search!
      ResultSet rs = stmt.executeQuery(sql.toString());

      if (!(rs.next()))
	throw new InternalStateError("getSearchCategoryCount search failure (MUST have 1 row!)");

      return rs.getInt("total");

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading SIG entries: " + e.getMessage(),e);
      throw new DataException("unable to retrieve user SIG information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end getSearchCategoryCount

} // end class CategoryDescriptorImpl
