/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Community System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.impl;

import java.sql.*;
import java.util.*;
import org.apache.log4j.*;
import com.silverwrist.venice.db.*;
import com.silverwrist.venice.security.AuditRecord;
import com.silverwrist.venice.core.*;

class ConferenceSIGContextImpl implements ConferenceSIGContext
{
  /*--------------------------------------------------------------------------------
   * Inner class used to cache locally-important data
   *--------------------------------------------------------------------------------
   */

  static class ConfCache
  {
    String name;
    String description;
    java.util.Date created;

    public ConfCache(String name, String description, java.util.Date created)
    {
      this.name = name;
      this.description = description;
      this.created = created;

    } // end constructor

    public String getName()
    {
      return name;

    } // end getName

    public String getDescription()
    {
      return description;

    } // end getDescription

    public java.util.Date getCreated()
    {
      return created;

    } // end getCreated

  } // end class ConfCache

  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(ConferenceSIGContextImpl.class.getName());

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private int refcount = 1;        // reference count (within the SIGData)
  private EngineBackend engine;    // engine object reference
  private SIGDataBackend sig;      // SIG object reference
  private DataPool datapool;       // data pool object
  private int confid;              // ID of this conference
  private int level;               // level granted in conference to members of the SIG
  private short sequence;          // sequence number this conference appears in list
  private boolean hide_list;       // hide the conference in the list?
  private ConfCache cache;         // cache of locally-important information
  private ConferenceData confdata = null;  // the inner conference data object

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  ConferenceSIGContextImpl(EngineBackend engine, SIGDataBackend sig, DataPool datapool, int confid)
      throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("new ConferenceSIGContextImpl(#" + String.valueOf(confid) + ") for SIG # "
		   + String.valueOf(sig.realSIGID()));

    this.engine = engine;
    this.sig = sig;
    this.datapool = datapool;
    this.confid = confid;

    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();

      // Build a monster query!  This is like the query for retrieving the ConferenceUserContextImpl
      // data, except without the left outer join to the user conference membership table.
      StringBuffer sql =
	  new StringBuffer("SELECT c.createdate, c.name, c.descr, s.granted_lvl, s.sequence, s.hide_list "
			   + "FROM sigtoconf s, confs c WHERE s.confid = c.confid AND s.sigid = ");
      sql.append(sig.realSIGID()).append(" AND c.confid = ").append(confid).append(';');
      if (logger.isDebugEnabled())
	logger.debug("SQL: " + sql.toString());

      // Run that monster query!
      ResultSet rs = stmt.executeQuery(sql.toString());
      if (!(rs.next()))
	throw new DataException("conference ID#" + String.valueOf(confid) + " not found in SIG#"
				+ String.valueOf(sig.realSIGID()));

      // fill in the "cache" and "level" indicators
      this.cache = new ConfCache(rs.getString(2),rs.getString(3),SQLUtil.getFullDateTime(rs,1));
      this.level = rs.getInt(4);
      this.sequence = rs.getShort(5);
      this.hide_list = rs.getBoolean(6);

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading conference entry: " + e.getMessage(),e);
      throw new DataException("unable to retrieve conference information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end constructor

  ConferenceSIGContextImpl(EngineBackend engine, SIGDataBackend sig, DataPool datapool,
			   short sequence, boolean hide_list, ConferenceData cdata)
  {
    if (logger.isDebugEnabled())
      logger.debug("new ConferenceSIGContextImpl(NEW#" + String.valueOf(confid) + ") for SIG # "
		   + String.valueOf(sig.realSIGID()));

    this.engine = engine;
    this.sig = sig;
    this.datapool = datapool;
    this.confid = cdata.getID();
    this.level = 0;
    this.sequence = sequence;
    this.hide_list = hide_list;
    this.cache = null;
    cdata.rd_addRef();
    this.confdata = cdata;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private synchronized ConferenceData getConferenceData() throws DataException
  {
    if (confdata==null)
    { // attempt to load the ConferenceSIGContext
      confdata = engine.getConferenceDataObject(confid);

      // clear cache when we get the real confdata
      cache = null;

    } // end if

    return confdata;

  } // end getConferenceData

  private synchronized ConferenceData getConferenceDataNE()
  {
    if (confdata==null)
    { // we need to load the ConferenceSIGContext...
      try
      { // attempt to load the ConferenceSIGContext
	confdata = engine.getConferenceDataObject(confid);

      } // end try
      catch (DataException e)
      { // there was a data failure loading the conference
	logger.error("** Got DataException in getConferenceDataNE: " + e.getMessage(),e);
	return null;

      } // end catch

      // clear cache when we get the real confdata
      cache = null;

    } // end if

    return confdata;

  } // end getConferenceDataNE

  /*--------------------------------------------------------------------------------
   * Implementations from interface ReferencedData
   *--------------------------------------------------------------------------------
   */

  public int rd_addRef()
  {
    return ++refcount;

  } // end rd_addRef

  public int rd_release()
  {
    return --refcount;

  } // end rd_release

  public boolean rd_unreferenced()
  {
    return (refcount<=0);

  } // end rd_unreferences

  /*--------------------------------------------------------------------------------
   * Implementations from interface ConferenceSIGContext
   *--------------------------------------------------------------------------------
   */

  public int getConfID()
  {
    return confid;

  } // end getConfID

  public String getName()
  {
    if (cache!=null)
      return cache.getName();
    ConferenceData c = getConferenceDataNE();
    if (c!=null)
      return c.getName();
    else
      return null;

  } // end getName
 
  public String getDescription()
  {
    if (cache!=null)
      return cache.getDescription();
    ConferenceData c = getConferenceDataNE();
    if (c!=null)
      return c.getDescription();
    else
      return null;

  } // end getDescription

  public java.util.Date getCreationDate()
  {
    if (cache!=null)
      return cache.getCreated();
    ConferenceData c = getConferenceDataNE();
    if (c!=null)
      return c.getCreationDate();
    else
      return null;

  } // end getCreationDate

  public java.util.Date getLastUpdateDate()
  {
    ConferenceData c = getConferenceDataNE();
    if (c!=null)
      return c.getLastUpdateDate();
    else
      return null;

  } // end getLastUpdateDate

  public int getSIGGrantedLevel()
  {
    return level;

  } // end getSIGGrantedLevel

  public List getAliases() throws DataException
  {
    return getConferenceData().getAliases();

  } // end getAliases
  
  public List getHosts() throws DataException
  {
    return getConferenceData().getHosts();

  } // end getHosts
  
  public boolean canReadConference(int level)
  {
    ConferenceData c = getConferenceDataNE();
    if (c==null)
      return false;
    if (level<this.level)
      return c.canReadConference(this.level);
    else
      return c.canReadConference(level);

  } // end canReadConference

  public boolean canPostToConference(int level)
  {
    ConferenceData c = getConferenceDataNE();
    if (c==null)
      return false;
    if (level<this.level)
      return c.canPostToConference(this.level);
    else
      return c.canPostToConference(level);

  } // end canPostToConference

  public boolean canCreateTopic(int level)
  {
    ConferenceData c = getConferenceDataNE();
    if (c==null)
      return false;
    if (level<this.level)
      return c.canCreateTopic(this.level);
    else
      return c.canCreateTopic(level);

  } // end canCreateTopic

  public boolean canChangeConference(int level)
  {
    ConferenceData c = getConferenceDataNE();
    if (c==null)
      return false;
    if (level<this.level)
      return c.canChangeConference(this.level);
    else
      return c.canChangeConference(level);

  } // end canChangeConference

  public int getReadLevel() throws DataException
  {
    return getConferenceData().getReadLevel();

  } // end getReadLevel

  public int getPostLevel() throws DataException
  {
    return getConferenceData().getPostLevel();

  } // end getPostLevel

  public int getCreateLevel() throws DataException
  {
    return getConferenceData().getCreateLevel();

  } // end getCreateLevel

  public int getHideLevel() throws DataException
  {
    return getConferenceData().getHideLevel();

  } // end getHideLevel

  public int getNukeLevel() throws DataException
  {
    return getConferenceData().getNukeLevel();

  } // end getNukeLevel

  public int getChangeLevel() throws DataException
  {
    return getConferenceData().getChangeLevel();

  } // end getChangeLevel

  public int getDeleteLevel() throws DataException
  {
    return getConferenceData().getDeleteLevel();

  } // end getDeleteLevel

  public synchronized void setSecurityLevels(SIGBackend sig, int read, int post, int create, int hide,
					     int nuke, int change, int delete) throws DataException
  {
    getConferenceData().setSecurityLevels(sig,read,post,create,hide,nuke,change,delete);

  } // end setSecurityLevels

  public synchronized void setName(SIGBackend sig, String val) throws DataException
  {
    getConferenceData().setName(sig,val);

  } // end setName

  public synchronized void setDescription(String val) throws DataException
  {
    getConferenceData().setDescription(val);

  } // end setDescription

  public synchronized void addAlias(SIGBackend sig, String alias) throws DataException
  {
    getConferenceData().addAlias(sig,alias);

  } // end addAlias

  public synchronized void removeAlias(SIGBackend sig, String alias) throws DataException
  {
    getConferenceData().removeAlias(sig,alias);

  } // end removeAlias

  public synchronized void setMembership(SIGBackend sig, int uid, int grant_level) throws DataException
  {
    getConferenceData().setMembership(sig,uid,grant_level);

  } // end setMembership

  public synchronized void setSIGGrantedLevel(SIGBackend sig, int new_level) throws DataException
  {
    Connection conn = null;  // database connection
    AuditRecord ar = null;   // audit record
      
    try
    { // get a database connection
      conn = datapool.getConnection();

      // create the SQL statement
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("UPDATE sigtoconf SET granted_lvl = ");
      sql.append(new_level).append(" WHERE sigid = ").append(this.sig.realSIGID()).append(" AND confid = ");
      sql.append(confid).append(';');

      // execute the update
      stmt.executeUpdate(sql.toString());

      // save copies of the data in our local variables
      level = new_level;

      // create an audit record reflecting the change
      ar = new AuditRecord(AuditRecord.CONF_SECURITY,sig.realUID(),sig.userRemoteAddress(),sig.realSIGID(),
			   "conf=" + String.valueOf(confid));

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error storing conference data: " + e.getMessage(),e);
      throw new DataException("unable to save conference data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end setSIGGrantedLevel

  public short getSequence()
  {
    return sequence;

  } // end getSequence

  public synchronized void setSequence(short seq) throws DataException
  {
    Connection conn = null;  // database connection

    try
    { // get a database connection
      conn = datapool.getConnection();

      // create the SQL statement
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("UPDATE sigtoconf SET sequence = ");
      sql.append(seq).append(" WHERE sigid = ").append(this.sig.realSIGID()).append(" AND confid = ");
      sql.append(confid).append(';');

      // execute the update
      stmt.executeUpdate(sql.toString());

      // save copies of the data in our local variables
      sequence = seq;

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error storing conference data: " + e.getMessage(),e);
      throw new DataException("unable to save conference data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end setSequence

  public boolean getHideList()
  {
    return hide_list;

  } // end getHideList

  public void setHideList(SIGBackend sig, boolean flag) throws DataException
  {
    Connection conn = null;  // database connection
    AuditRecord ar = null;   // audit record
      
    try
    { // get a database connection
      conn = datapool.getConnection();

      // create the SQL statement
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("UPDATE sigtoconf SET hide_list = ");
      sql.append(hide_list ? '1' : '0').append(" WHERE sigid = ").append(this.sig.realSIGID());
      sql.append(" AND confid = ").append(confid).append(';');

      // execute the update
      stmt.executeUpdate(sql.toString());

      // save copies of the data in our local variables
      hide_list = flag;

      // create an audit record reflecting the change
      ar = new AuditRecord(AuditRecord.CONF_SECURITY,sig.realUID(),sig.userRemoteAddress(),sig.realSIGID(),
			   "conf=" + String.valueOf(confid));

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error storing conference data: " + e.getMessage(),e);
      throw new DataException("unable to save conference data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end setHideList

  public boolean canHideTopics(int level)
  {
    ConferenceData c = getConferenceDataNE();
    if (c==null)
      return false;
    if (level<this.level)
      return c.canHideTopics(this.level);
    else
      return c.canHideTopics(level);

  } // end canHideTopics

  public String getAnAlias() throws DataException
  {
    return getConferenceData().getAnAlias();

  } // end getAnAlias

  public ReturnTopicInfo createNewTopic(SIGBackend sig, String title, String pseud, String body,
					int body_lines) throws DataException
  {
    return getConferenceData().createNewTopic(sig,title,pseud,body,body_lines);

  } // end createNewTopic

  public boolean canScribblePosts(int level)
  {
    ConferenceData c = getConferenceDataNE();
    if (c==null)
      return false;
    if (level<this.level)
      return c.canScribblePosts(this.level);
    else
      return c.canScribblePosts(level);

  } // end canScribblePosts

  public boolean canNukePosts(int level)
  {
    ConferenceData c = getConferenceDataNE();
    if (c==null)
      return false;
    if (level<this.level)
      return c.canNukePosts(this.level);
    else
      return c.canNukePosts(level);

  } // end canNukePosts

  public void touchUpdate(Connection conn, java.util.Date date) throws DataException
  {
    getConferenceData().touchUpdate(conn,date);

  } // end touchUpdate

} // end class ConferenceSIGContextImpl
