/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Community System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets;

import java.io.*;
import java.net.URLEncoder;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.*;
import com.silverwrist.util.StringUtil;
import com.silverwrist.venice.ValidationException;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.servlets.format.*;

public class Account extends VeniceServlet
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(Account.class.getName());

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private NewAccountDialog makeNewAccountDialog() throws ServletException
  {
    final String desired_name = "NewAccountDialog";
    DialogCache cache = DialogCache.getDialogCache(getServletContext());

    if (!(cache.isCached(desired_name)))
    { // create a template and save it off
      NewAccountDialog template = new NewAccountDialog(getCountryList());
      cache.saveTemplate(template);

    } // end if

    // return a new copy
    return (NewAccountDialog)(cache.getNewDialog(desired_name));

  } // end makeNewAccountDialog

  private LoginDialog makeLoginDialog()
  {
    final String desired_name = "LoginDialog";
    DialogCache cache = DialogCache.getDialogCache(getServletContext());

    if (!(cache.isCached(desired_name)))
    { // create a template and save it off
      LoginDialog template = new LoginDialog();
      cache.saveTemplate(template);

    } // end if

    // return a new copy
    return (LoginDialog)(cache.getNewDialog(desired_name));

  } // end makeLoginDialog

  private VerifyEmailDialog makeVerifyEmailDialog()
  {
    final String desired_name = "VerifyEmailDialog";
    DialogCache cache = DialogCache.getDialogCache(getServletContext());

    if (!(cache.isCached(desired_name)))
    { // create a template and save it off
      VerifyEmailDialog template = new VerifyEmailDialog();
      cache.saveTemplate(template);

    } // end if

    // return a new copy
    return (VerifyEmailDialog)(cache.getNewDialog(desired_name));

  } // end makeVerifyEmailDialog

  private EditProfileDialog makeEditProfileDialog() throws ServletException
  {
    final String desired_name = "EditProfileDialog";
    DialogCache cache = DialogCache.getDialogCache(getServletContext());

    if (!(cache.isCached(desired_name)))
    { // create a template and save it off
      EditProfileDialog template = new EditProfileDialog(getCountryList());
      cache.saveTemplate(template);

    } // end if

    // return a new copy
    return (EditProfileDialog)(cache.getNewDialog(desired_name));

  } // end makeEditProfileDialog

  /*--------------------------------------------------------------------------------
   * Overrides from class HttpServlet
   *--------------------------------------------------------------------------------
   */

  public String getServletInfo()
  {
    String rc = "Account servlet - Handles login, logout, account creation, and profiles\n"
              + "Part of the Venice Web Communities System\n";
    return rc;

  } // end getServletInfo

  public void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    RenderData rdat = createRenderData(request,response);
    UserContext user = getUserContext(request);
    String page_title = null;
    ContentRender content = null;
    boolean display_login = false;

    String tgt = request.getParameter("tgt");  // target location
    if (tgt==null)
      tgt = "top";  // go back to the Top screen if nothing else
    String cmd = request.getParameter("cmd");  // command parameter

    if (cmd.equals("L"))
    { // "L" = Log in/out
      if (user.isLoggedIn())
      { // log the user out and send 'em back to the "top" page
	clearUserContext(request);
	// TODO: here is where the persistent cookie gets nuked, if it does...
	rdat.redirectTo("top");
	return;

      } // end if (logging out)
      else
      { // display the Login page
	LoginDialog dlg = makeLoginDialog();
	dlg.setupNew(tgt);
	page_title = dlg.getTitle();
	content = dlg;

      } // end else

    } // end if ("L" command)
    else if (cmd.equals("C"))
    { // "C" = create new account
      if (user.isLoggedIn())
      { // you can't create a new account while logged in on an existing one!
	page_title = "Error";
	content = new ErrorBox(page_title,"You cannot create a new account while logged in on "
			       + "an existing one.  You must log out first.",tgt);

      } // end if
      else
      { // display the "Create Account" page
	NewAccountDialog dlg = makeNewAccountDialog();
	dlg.setEngine(getVeniceEngine());
	page_title = dlg.getTitle();
	dlg.setTarget(tgt);
	dlg.setFieldValue("country","US");
	content = dlg;

      } // end else

    } // end if ("C" command)
    else if (cmd.equals("P"))
    { // "P" = user profile
      if (user.isLoggedIn())
      { // display the User Profile page
	EditProfileDialog dlg = makeEditProfileDialog();
	try
	{ // load the profile information
	  dlg.setupDialog(user,tgt);

	  // prepare for display
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end try
	catch (DataException e)
	{ // we couldn't load the contact info for the profile
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error retrieving profile: " + e.getMessage(),tgt);

	} // end catch

      } // end if
      else
      { // you have to be logged in for this one!
	page_title = "Error";
	content = new ErrorBox(page_title,"You must log in before you can modify the profile "
			       + "on your account.",tgt);

      } // end else

    } // end if ("P" command)
    else if (cmd.equals("V"))
    { // "V" = verify email address
      display_login = true;

      if (user.isLoggedIn())
      { // display the Verify E-mail Address page
	VerifyEmailDialog dlg = makeVerifyEmailDialog();
	page_title = dlg.getTitle();
	dlg.setTarget(tgt);
	content = dlg;

      } // end if
      else
      { // you have to be logged in for this one!
	page_title = "Error";
	content = new ErrorBox(page_title,"You must log in before you can verify your account's "
			       + "email address.",tgt);

      } // end else

    } // end else if ("V" command)
    else
    { // unknown command
      page_title = "Internal Error";
      logger.error("invalid command to Account.doGet: " + cmd);
      content = new ErrorBox(page_title,"Invalid command to Account.doGet",tgt);

    } // end else

    BaseJSPData basedat = new BaseJSPData(page_title,"account?cmd=" + cmd,content);
    basedat.displayLoginLinks(display_login);
    basedat.transfer(getServletContext(),rdat);

  } // end doGet

  public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    RenderData rdat = createRenderData(request,response);
    UserContext user = getUserContext(request);
    String page_title = null;
    ContentRender content = null;
    boolean display_login = false;

    String tgt = request.getParameter("tgt");  // target location
    if (tgt==null)
      tgt = "top";  // go back to Top screen if nothing else
    String cmd = request.getParameter("cmd");  // command parameter

    if (cmd.equals("L"))
    { // log the user in
      LoginDialog dlg = makeLoginDialog();

      if (dlg.isButtonClicked(request,"cancel"))
      { // go back where we came from - we decided not to log in anyhoo
	rdat.redirectTo(tgt);
	return;

      } // end if (canceled)

      if (dlg.isButtonClicked(request,"remind"))
      { // this will email a password reminder to the owner's account
	dlg.loadValues(request);  // load the dialog values
	try
	{ // send the password reminder
	  getVeniceEngine().sendPasswordReminder(dlg.getFieldValue("user"));

	  // recycle and redisplay the dialog box
	  dlg.resetOnError("Password reminder has been sent to your e-mail address.");
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end try
	catch (DataException e1)
	{ // this indicates a database error - display an ErrorBox
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error finding user: " + e1.getMessage(),tgt);

	} // end catch
	catch (AccessError e2)
	{ // this indicates a problem with the user account or password
	  page_title = "User E-mail Address Not Found";
	  content = new ErrorBox(page_title,e2.getMessage(),tgt);

	} // end catch
	catch (EmailException ee)
	{ // error sending the confirmation email
	  page_title = "E-mail Error";
	  content = new ErrorBox(page_title,"E-mail error sending reminder: " + ee.getMessage(),tgt);

	} // end catch

      } // end if ("reminder" button clicked)
      else if (dlg.isButtonClicked(request,"login"))
      { // we actually want to try and log in!  imagine that!
	dlg.loadValues(request);  // load the values

	try
	{ // use the user context to authenticate
	  user.authenticate(dlg.getFieldValue("user"),dlg.getFieldValue("pass"));

	  // TODO: here is where the persistent cookie gets sent, if it does...

	  // assuming it worked OK, redirect them back where they came from
	  // (or to the verification page if they need to go there)
	  String url;
	  if (user.isEmailVerified())
	    url = tgt;
	  else // they haven't verified yet - remind them to do so
	    url = "account?cmd=V&tgt=" + URLEncoder.encode(tgt);

	  clearMenu(request);
	  rdat.redirectTo(url);
	  return;

	} // end try
	catch (DataException e1)
	{ // this indicates a database error - display an ErrorBox
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error logging in: " + e1.getMessage(),tgt);

	} // end catch
	catch (AccessError e2)
	{ // this indicates a problem with the user account or password
	  dlg.resetOnError(e2.getMessage());
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end catch

      } // end else if ("login" button clicked)
      else
      { // the button must be wrong!
	page_title = "Internal Error";
	logger.error("no known button click on Account.doPost, cmd=L");
	content = new ErrorBox(page_title,"Unknown command button pressed",tgt);

      } // end else

    } // end if ("L" command)
    else if (cmd.equals("C"))
    { // C = Create New Account
      NewAccountDialog dlg = makeNewAccountDialog();
      dlg.setEngine(getVeniceEngine());

      if (dlg.isButtonClicked(request,"cancel"))
      { // go back where we came from - we decided not to create the account anyhoo
	String url = response.encodeRedirectURL(rdat.getFullServletPath(tgt));
	response.sendRedirect(url);
	return;

      } // end if ("cancel" button clicked)

      if (dlg.isButtonClicked(request,"create"))
      { // OK, actually want to create the account!
	dlg.loadValues(request);  // load field values
      
	try
	{ // attempt to create the account
	  UserContext uc = dlg.doDialog(request);
	  putUserContext(request,uc);  // effectively logging in the new user
	  user = uc;

	  // now jump to the Verify page
	  String partial = "account?cmd=V&tgt=" + URLEncoder.encode(tgt);
	  String url = response.encodeRedirectURL(rdat.getFullServletPath(partial));
	  response.sendRedirect(url);
	  return;

	} // end try
	catch (ValidationException ve)
	{ // unable to validate some of the data in the form
	  dlg.resetOnError(ve.getMessage() + " Please try again.");
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end catch
	catch (DataException de)
	{ // data error in setting up account
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error creating account: " + de.getMessage(),tgt);

	} // end catch
	catch (AccessError ae)
	{ // access error in setting up the account
	  dlg.resetOnError(ae.getMessage());
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end catch
	catch (EmailException ee)
	{ // error sending the confirmation email
	  page_title = "E-mail Error";
	  content = new ErrorBox(page_title,"E-mail error creating account: " + ee.getMessage(),tgt);

	} // end catch

      } // end if ("create" button clicked)
      else
      { // the button must be wrong!
	page_title = "Internal Error";
	logger.error("no known button click on Account.doPost, cmd=C");
	content = new ErrorBox(page_title,"Unknown command button pressed",tgt);

      } // end else

    } // end else if ("C" command)
    else if (cmd.equals("V"))
    { // V = Verify E-mail Address
      display_login = true;
      VerifyEmailDialog dlg = makeVerifyEmailDialog();

      if (dlg.isButtonClicked(request,"cancel"))
      { // go back to our desired location
	String url = response.encodeRedirectURL(rdat.getFullServletPath(tgt));
	response.sendRedirect(url);
	return;

      } // end if ("cancel" button clicked)

      if (dlg.isButtonClicked(request,"again"))
      { // do a "send again"
	try
	{ // resend the email confirmation
	  user.resendEmailConfirmation();

	  // now force the form to be redisplayed
	  dlg.clearAllFields();
	  dlg.setTarget(tgt);
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end try
	catch (DataException de)
	{ // data error in setting up account
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error sending confirmation: " + de.getMessage(),tgt);

	} // end catch
	catch (EmailException ee)
	{ // error sending the confirmation email
	  page_title = "E-mail Error";
	  content = new ErrorBox(page_title,"E-mail error sending confirmation: " + ee.getMessage(),tgt);

	} // end catch

      } // end if ("again" clicked)
      else if (dlg.isButtonClicked(request,"ok"))
      { // try to verify the confirmation number
	dlg.loadValues(request);  // load field values

	try
	{ // validate that we have a good number
	  dlg.validate();
	  user.confirmEmail(dlg.getConfirmationNumber());

	  // go back to our desired location
	  String url = response.encodeRedirectURL(rdat.getFullServletPath(tgt));
	  response.sendRedirect(url);
	  return;

	} // end try
	catch (ValidationException ve)
	{ // there was a validation error...
	  dlg.resetOnError(ve.getMessage() + " Please try again.");
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end catch
	catch (DataException de)
	{ // data error in setting up account
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error verifying email: " + de.getMessage(),tgt);

	} // end catch
	catch (AccessError ae)
	{ // access error in setting up the account
	  dlg.resetOnError(ae.getMessage());
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end catch

      } // end else if ("OK" clicked)
      else
      { // the button must be wrong!
	page_title = "Internal Error";
	logger.error("no known button click on Account.doPost, cmd=V");
	content = new ErrorBox(page_title,"Unknown command button pressed",tgt);

      } // end else

    } // end else if ("V" command)
    else if (cmd.equals("P"))
    { // P = Edit User Profile
      display_login = true;
      EditProfileDialog dlg = makeEditProfileDialog();

      if (dlg.isButtonClicked(request,"cancel"))
      { // go back to our desired location
	String url = response.encodeRedirectURL(rdat.getFullServletPath(tgt));
	response.sendRedirect(url);
	return;

      } // end if ("cancel" button clicked)

      if (dlg.isButtonClicked(request,"update"))
      { // we're ready to update the user profile
	dlg.loadValues(request);  // load field values

	try
	{ // validate the dialog and start setting info
	  String url;
	  if (dlg.doDialog(user))
	  { // the email address was changed - need to jump to the "Verify" page
	    String partial = "account?cmd=V&tgt=" + URLEncoder.encode(tgt);
	    url = response.encodeRedirectURL(rdat.getFullServletPath(partial));

	  } // end if
	  else  // just go back where we came from
	    url = response.encodeRedirectURL(rdat.getFullServletPath(tgt));

	  response.sendRedirect(url);
	  return;

	} // end try
	catch (ValidationException ve)
	{ // there was a validation error...
	  dlg.resetOnError(ve.getMessage() + " Please try again.");
	  page_title = dlg.getTitle();
	  content = dlg;

	} // end catch
	catch (DataException de)
	{ // data error in setting up account
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error updating profile: " + de.getMessage(),tgt);

	} // end catch
	catch (EmailException ee)
	{ // error sending the confirmation email
	  page_title = "E-mail Error";
	  content = new ErrorBox(page_title,"E-mail error sending confirmation: " + ee.getMessage(),tgt);

	} // end catch

      } // end if ("update" button pressed)
      else
      { // the button must be wrong!
	page_title = "Internal Error";
	logger.error("no known button click on Account.doPost, cmd=P");
	content = new ErrorBox(page_title,"Unknown command button pressed",tgt);

      } // end else

    } // end else if ("P" command)
    else
    { // unknown command
      page_title = "Internal Error";
      logger.error("invalid command to Account.doPost: " + cmd);
      content = new ErrorBox(page_title,"Invalid command to Account.doPost",tgt);

    } // end else

    changeMenuTop(request);

    BaseJSPData basedat = new BaseJSPData(page_title,"account?cmd=" + cmd,content);
    basedat.displayLoginLinks(display_login);
    basedat.transfer(getServletContext(),rdat);

  } // end doPost

} // end class Account
