/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.*;
import com.silverwrist.util.StringUtil;
import com.silverwrist.venice.ValidationException;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.servlets.format.*;

public class PostMessage extends VeniceServlet
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(PostMessage.class.getName());

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private static SIGContext getSIGParameter(ServletRequest request, UserContext user)
      throws ValidationException, DataException
  {
    String str = request.getParameter("sig");
    if (str==null)
    { // no SIG parameter - bail out now!
      logger.error("SIG parameter not specified!");
      throw new ValidationException("No SIG specified.");

    } // end if

    try
    { // turn the string into a SIGID, and thence to a SIGContext
      int sigid = Integer.parseInt(str);
      return user.getSIGContext(sigid);

    } // end try
    catch (NumberFormatException nfe)
    { // error in Integer.parseInt
      logger.error("Cannot convert SIG parameter '" + str + "'!");
      throw new ValidationException("Invalid SIG parameter.");

    } // end catch

  } // end getSIGParameter

  private static ConferenceContext getConferenceParameter(ServletRequest request, SIGContext sig)
    throws ValidationException, DataException, AccessError
  {
    String str = request.getParameter("conf");
    if (str==null)
    { // no conference parameter - bail out now!
      logger.error("Conference parameter not specified!");
      throw new ValidationException("No conference specified.");

    } // end if

    try
    { // turn the string into a ConfID, and thence to a ConferenceContext
      int confid = Integer.parseInt(str);
      return sig.getConferenceContext(confid);

    } // end try
    catch (NumberFormatException nfe)
    { // error in Integer.parseInt
      logger.error("Cannot convert conference parameter '" + str + "'!");
      throw new ValidationException("Invalid conference parameter.");

    } // end catch

  } // end getConferenceParameter

  private static TopicContext getTopicParameter(ServletRequest request, ConferenceContext conf)
    throws ValidationException, DataException, AccessError
  {
    String str = request.getParameter("top");
    if (StringUtil.isStringEmpty(str))
    { // no topic parameter - bail out now!
      logger.error("Topic parameter not specified!");
      throw new ValidationException("No topic specified.");

    } // end if

    try
    { // turn the string into a TopicID, and thence to a TopicContext
      short topicid = Short.parseShort(str);
      return conf.getTopic(topicid);

    } // end try
    catch (NumberFormatException nfe)
    { // error in Integer.parseInt
      logger.error("Cannot convert topic parameter '" + str + "'!");
      throw new ValidationException("Invalid topic parameter.");

    } // end catch

  } // end getTopicParameter

  private static int getPostNumber(ServletRequest request) throws ValidationException
  {
    String str = request.getParameter("sd");
    if (StringUtil.isStringEmpty(str))
      throw new ValidationException("Invalid parameter.");
    try
    { // get the number of posts we think he topic has
      return Integer.parseInt(str);

    } // end try
    catch (NumberFormatException nfe)
    { // not a good integer...
      throw new ValidationException("Invalid parameter.");

    } // end catch

  } // end getPostNumber

  /*--------------------------------------------------------------------------------
   * Overrides from class HttpServlet
   *--------------------------------------------------------------------------------
   */

  public String getServletInfo()
  {
    String rc = "PostMessage servlet - Handles posting messages to a conference\n"
              + "Part of the Venice Web Communities System\n";
    return rc;

  } // end getServletInfo

  public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    UserContext user = getUserContext(request);
    RenderData rdat = createRenderData(request,response);
    String page_title = null;
    Object content = null;
    SIGContext sig = null;            // SIG context
    ConferenceContext conf = null;    // conference context
    TopicContext topic = null;        // topic context

    try
    { // this outer try is to catch ValidationException
      try
      { // all commands require a SIG parameter
	sig = getSIGParameter(request,user);
	changeMenuSIG(request,sig);
	if (logger.isDebugEnabled())
	  logger.debug("found SIG #" + String.valueOf(sig.getSIGID()));

      } // end try
      catch (DataException de)
      { // error looking up the SIG
	page_title = "Database Error";
	content = new ErrorBox(page_title,"Database error finding SIG: " + de.getMessage(),"top");

      } // end catch

      if (content==null)
      { // we got the SIG parameter OK
	try
	{ // all commands require a conference parameter
	  conf = getConferenceParameter(request,sig);
	  if (logger.isDebugEnabled())
	    logger.debug("found conf #" + String.valueOf(conf.getConfID()));
	  
	} // end try
	catch (DataException de)
	{ // error looking up the conference
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error finding conference: " + de.getMessage(),"top");

	} // end catch

      } // end if

      if (content==null)
      { // we got the conference parameter OK
	try
	{ // now we need a topic parameter
	  topic = getTopicParameter(request,conf);
	  if (logger.isDebugEnabled())
	    logger.debug("found topic #" + String.valueOf(topic.getTopicID()));

	} // end try
	catch (DataException de)
	{ // error looking up the conference
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error finding topic: " + de.getMessage(),"top");

	} // end catch

      } // end if

    } // end try
    catch (ValidationException ve)
    { // these all get handled in pretty much the same way
      page_title = "Error";
      content = new ErrorBox(null,ve.getMessage(),"top");

    } // end catch
    catch (AccessError ae)
    { // these all get handled in pretty much the same way
      page_title = "Access Error";
      content = new ErrorBox(page_title,ae.getMessage(),"top");

    } // end catch

    if (content==null)
    { // make sure we've got some post data
      String raw_postdata = request.getParameter("pb");
      if (StringUtil.isStringEmpty(raw_postdata))
      { // don't allow zero-size posts
	rdat.nullResponse();
	return;

      } // end if

      final String yes = "Y";

      // now decide what to do based on which button got clicked
      if (isImageButtonClicked(request,"cancel"))
      { // canceled posting - take us back to familiar ground
	rdat.redirectTo("confdisp?sig=" + String.valueOf(sig.getSIGID()) + "&conf="
                        + String.valueOf(conf.getConfID()) + "&top="
                        + String.valueOf(topic.getTopicNumber()));
	return;

      } // end if ("Cancel")
      else if (isImageButtonClicked(request,"preview"))
      { // previewing the post!
	try
	{ // generate a preview view
	  content = new PostPreview(getVeniceEngine(),sig,conf,topic,request.getParameter("pseud"),
                                    raw_postdata,request.getParameter("next"),getPostNumber(request),
				    yes.equals(request.getParameter("attach")));
	  page_title = "Previewing Post";

	} // end try
	catch (ValidationException ve)
	{ // there was some sort of a parameter error in the display (getPostNumber can throw this)
	  page_title = "Error";
	  content = new ErrorBox(null,ve.getMessage(),"top");

	} // end catch

      } // end else if ("Preview & Spellcheck")
      else if (isImageButtonClicked(request,"post"))
      { // post the message, and then reload the same topic
	try
	{ // first, check against slippage
	  int pn = getPostNumber(request);
	  if (pn==topic.getTotalMessages())
	  { // no slippage - post the message!!!
	    TopicMessageContext msg = topic.postNewMessage(0,request.getParameter("pseud"),raw_postdata);
	    if (yes.equals(request.getParameter("attach")))
	    { // we have an attachment to upload...
	      // TODO: do something to upload the attachment
	      rdat.redirectTo("confdisp?sig=" + String.valueOf(sig.getSIGID()) + "&conf="
			      + String.valueOf(conf.getConfID()) + "&top="
			      + String.valueOf(topic.getTopicNumber()) + "&rnm=1");
	      return;

	    } // end if
	    else
	    { // no attachment - jump back to the topic
	      rdat.redirectTo("confdisp?sig=" + String.valueOf(sig.getSIGID()) + "&conf="
			      + String.valueOf(conf.getConfID()) + "&top="
			      + String.valueOf(topic.getTopicNumber()) + "&rnm=1");
	      return;

	    } // end else

	  } // end if
	  else
	  { // slippage detected - show the slippage display
	    content = new PostSlippage(getVeniceEngine(),sig,conf,topic,pn,request.getParameter("next"),
				       request.getParameter("pseud"),raw_postdata,
                                       yes.equals(request.getParameter("attach")));
	    page_title = "Slippage or Double-Click Detected";

	  } // end else

	} // end try
	catch (ValidationException ve)
	{ // there was some sort of a parameter error in the display
	  page_title = "Error";
	  content = new ErrorBox(null,ve.getMessage(),"top");

	} // end catch
	catch (DataException de)
	{ // there was a database error posting the message
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error posting message: " + de.getMessage(),"top");

	} // end catch
	catch (AccessError ae)
	{ // we were unable to retrieve the topic list
	  page_title = "Access Error";
	  content = new ErrorBox(page_title,ae.getMessage(),"top");

	} // end catch

      } // end else if ("Post & Reload")
      else if (isImageButtonClicked(request,"postnext"))
      { // post the message, and then go to the "next" topic
	try
	{ // first, check against slippage
	  int pn = getPostNumber(request);
	  if (pn==topic.getTotalMessages())
	  { // no slippage - post the message!
	    TopicMessageContext msg = topic.postNewMessage(0,request.getParameter("pseud"),raw_postdata);

	    short next;
	    try
	    { // attempt to get the value of the "next topic" parameter
	      String foo = request.getParameter("next");
	      if (StringUtil.isStringEmpty(foo))
		next = topic.getTopicNumber();
	      else
		next = Short.parseShort(foo);

	    } // end try
	    catch (NumberFormatException nfe)
	    { // just default me
	      next = topic.getTopicNumber();

	    } // end catch

	    if (yes.equals(request.getParameter("attach")))
	    { // we have an attachment to upload...
	      // TODO: jump somewhere we can upload the attachment!
	      rdat.redirectTo("confdisp?sig=" + String.valueOf(sig.getSIGID()) + "&conf="
			      + String.valueOf(conf.getConfID()) + "&top=" + String.valueOf(next) + "&rnm=1");
	      return;

	    } // end if
	    else
	    { // no attachment - jump to the next topic
	      rdat.redirectTo("confdisp?sig=" + String.valueOf(sig.getSIGID()) + "&conf="
			      + String.valueOf(conf.getConfID()) + "&top=" + String.valueOf(next) + "&rnm=1");
	      return;

	    } // end else

	  } // end if
	  else
	  { // slippage detected - show the slippage display
	    content = new PostSlippage(getVeniceEngine(),sig,conf,topic,pn,request.getParameter("next"),
				       request.getParameter("pseud"),raw_postdata,
                                       yes.equals(request.getParameter("attach")));
	    page_title = "Slippage or Double-Click Detected";

	  } // end else

	} // end try
	catch (ValidationException ve)
	{ // there was some sort of a parameter error in the display
	  page_title = "Error";
	  content = new ErrorBox(null,ve.getMessage(),"top");

	} // end catch
	catch (DataException de)
	{ // there was a database error posting the message
	  page_title = "Database Error";
	  content = new ErrorBox(page_title,"Database error posting message: " + de.getMessage(),"top");

	} // end catch
	catch (AccessError ae)
	{ // we were unable to retrieve the topic list
	  page_title = "Access Error";
	  content = new ErrorBox(page_title,ae.getMessage(),"top");

	} // end catch

      } // end else if ("Post & Go Next")
      else
      { // unknown button clicked
	page_title = "Internal Error";
	logger.error("no known button click on PostMessage.doPost");
	content = new ErrorBox(page_title,"Unknown command button pressed","top");

      } // end else

    } // end if (got all parameters oK)

    BaseJSPData basedat = new BaseJSPData(page_title,"post",content);
    basedat.transfer(getServletContext(),rdat);

  } // end doPost

} // end class PostMessage
