/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Community System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.*;
import com.silverwrist.util.StringUtil;
import com.silverwrist.venice.ValidationException;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.servlets.format.*;

public class SIGAdmin extends VeniceServlet
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(SIGAdmin.class.getName());

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private SIGAdminTop makeSIGAdminTop() throws ServletException
  {
    final String desired_name = "SIGAdminTop";
    MenuPanelCache cache = MenuPanelCache.getMenuPanelCache(getServletContext());

    if (!(cache.isCached(desired_name)))
    { // create a template and save it off
      SIGAdminTop template = new SIGAdminTop();
      cache.saveTemplate(template);

    } // end if

    // return a new copy
    return (SIGAdminTop)(cache.getNewMenuPanel(desired_name));

  } // end makeSIGAdminTop

  private EditSIGProfileDialog makeEditSIGProfileDialog() throws ServletException
  {
    final String desired_name = "EditSIGProfileDialog";
    DialogCache cache = DialogCache.getDialogCache(getServletContext());

    if (!(cache.isCached(desired_name)))
    { // create a template and save it off
      EditSIGProfileDialog template = new EditSIGProfileDialog(getCountryList(),getLanguageList());
      cache.saveTemplate(template);

    } // end if

    // return a new copy
    return (EditSIGProfileDialog)(cache.getNewDialog(desired_name));

  } // end makeEditSIGProfileDialog

  /*--------------------------------------------------------------------------------
   * Overrides from class HttpServlet
   *--------------------------------------------------------------------------------
   */

  public String getServletInfo()
  {
    String rc = "SIGAdmin servlet - Administrative functions for SIGs\n"
              + "Part of the Venice Web Communities System\n";
    return rc;

  } // end getServletInfo

  public void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    UserContext user = getUserContext(request);
    RenderData rdat = createRenderData(request,response);
    String page_title = null;
    Object content = null;

    SIGContext sig = null;
    try
    { // first get the SIG context we're working with
      int sigid = Integer.parseInt(request.getParameter("sig"));
      sig = user.getSIGContext(sigid);
      changeMenuSIG(request,sig);
      
    } // end try
    catch (NumberFormatException nfe)
    { // an improperly formatted SIGID brought us down!
      page_title = "Input Error";
      logger.error("Somebody fed 'sig=" + request.getParameter("sig") + "' to this page, that's bogus");
      content = new ErrorBox(page_title,"SIG ID not valid: " + request.getParameter("sig"),"top");

    } // end catch
    catch (DataException de)
    { // unable to pull SIG data out of the database
      page_title = "Database Error";
      logger.error("database error looking up SIGID " + request.getParameter("sig"));
      content = new ErrorBox(page_title,"Database error accessing SIG: " + de.getMessage(),"top");

    } // end catch

    if (logger.isDebugEnabled() && (sig!=null))
      logger.debug("SIGAdmin/doGet operating on SIG \"" + sig.getName() + "\" ("
		   + String.valueOf(sig.getSIGID()) + ")");

    if (content==null)
    { // now decide what to do based on the "cmd" parameter
      String cmd = request.getParameter("cmd");
      if (cmd==null)
	cmd = "???";  // something pretty much guaranteed not to be used

      if (logger.isDebugEnabled())
	logger.debug("SIGAdmin/doGet command value = " + cmd);

      if (cmd.equals("P"))
      { // this is the profile editing screen
	if (sig.canModifyProfile())
	{ // construct the edit profile dialog and load it up for use
	  EditSIGProfileDialog dlg = makeEditSIGProfileDialog();

	  try
	  { // load the values for this dialog
	    dlg.setupDialog(getVeniceEngine(),sig);

	    // prepare for display
	    page_title = dlg.getTitle();
	    content = dlg;

	  } // end try
	  catch (DataException de)
	  { // we could not load the values because of a data exception
	    page_title = "Database Error";
	    logger.error("DB error loading SIG profile: " + de.getMessage(),de);
	    content = new ErrorBox(page_title,"Database error retrieving profile: " + de.getMessage(),
				   "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	  } // end catch
	  catch (AccessError ae)
	  { // we don't have enough privilege
	    page_title = "Unauthorized";
	    logger.error("Access error loading SIG profile: " + ae.getMessage(),ae);
	    content = new ErrorBox(page_title,"You do not have access to modify this SIG's profile.",
				   "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	  } // end catch

	} // end if
	else
	{ // no access - sorry, dude
	  page_title = "Unauthorized";
	  logger.error("tried to call up SIG profile screen without access...naughty naughty!");
	  content = new ErrorBox(page_title,"You do not have access to modify this SIG's profile.",
				 "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	} // end else

      } // end if (profile editing)
      else if (cmd.equals("T"))
      { // this is the category set code!
	if (sig.canModifyProfile())
	{ // did they actually send a "set" parameter?
	  String p = request.getParameter("set");
	  if (!(StringUtil.isStringEmpty(p)))
	  { // OK, we're setting the category ID...
	    try
	    { // get the new category ID and set it
	      int catid = Integer.parseInt(p);
	      if (!(getVeniceEngine().isValidCategoryID(catid)))
		throw new NumberFormatException();  // dump the category if it's not valid

	      sig.setCategoryID(catid);

	      // now that the category ID is set, go back to the admin menu
	      String target = "sigadmin?sig=" + String.valueOf(sig.getSIGID());
	      String url = response.encodeRedirectURL(rdat.getFullServletPath(target));
	      response.sendRedirect(url);
	      return;

	    } // end try
	    catch (NumberFormatException nfe)
	    { // we got an invalid category value...
	      page_title = "Invalid Input";
	      content = new ErrorBox(page_title,"Invalid category ID passed to category browser.",
				     "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	    } // end catch
	    catch (DataException de)
	    { // unable to update the SIG properly
	      page_title = "Database Error";
	      logger.error("DB error updating SIG: " + de.getMessage(),de);
	      content = new ErrorBox(page_title,"Database error updating SIG: " + de.getMessage(),
				     "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	    } // end catch
	    catch (AccessError ae)
	    { // database access error - display an error box
	      page_title = "Access Error";
	      logger.error("Access error updating SIG: " + ae.getMessage(),ae);
	      content = new ErrorBox(page_title,ae.getMessage(),
				     "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	    } // end catch

	  } // end if (actually setting the category
	  else
	  { // we're browsing - try and figure out what to display
	    try
	    { // get the "go" parameter to see what the current category ID is
	      p = request.getParameter("go");
	      int curr_catid;
	      if (StringUtil.isStringEmpty(p))
		curr_catid = sig.getCategoryID();
	      else
		curr_catid = Integer.parseInt(p);

	      if (!(getVeniceEngine().isValidCategoryID(curr_catid)))
		throw new NumberFormatException();  // dump the category if it's not valid

	      // create the browser panel and let it rip
	      content = new SIGCategoryBrowseData(user,sig,curr_catid);
	      page_title = "Set SIG Category";

	    } // end try
	    catch (NumberFormatException nfe)
	    { // we got an invalid category value...
	      page_title = "Invalid Input";
	      content = new ErrorBox(page_title,"Invalid category ID passed to category browser.",
				     "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	    } // end catch
	    catch (DataException de)
	    { // unable to get the categories properly
	      page_title = "Database Error";
	      logger.error("DB error browsing categories: " + de.getMessage(),de);
	      content = new ErrorBox(page_title,"Database error browsing categories: " + de.getMessage(),
				     "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	    } // end catch

	  } // end else (browsing through categories)

	} // end if
	else
	{ // no access - sorry man
	  page_title = "Unauthorized";
	  logger.error("tried to call up SIG category set screen without access...naughty naughty!");
	  content = new ErrorBox(page_title,"You do not have access to modify this SIG's profile.",
				 "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	} // end else

      } // end else if (category selector)
      else // other request
      { // all unknown requests get turned into menu display requests
	if (sig.canAdministerSIG())
	{ // create a SIG administration menu
	  SIGAdminTop menu = makeSIGAdminTop();
	  menu.setSIG(sig);
	  content = menu;
	  page_title = "SIG Administration";

	} // end if
	else
	{ // no access - sorry buddy
	  page_title = "Access Error";
	  logger.error("tried to call up SIG admin menu without access...naughty naughty!");
	  content = new ErrorBox(page_title,"You do not have access to administer this SIG.",
				 "sigprofile?sig=" + String.valueOf(sig.getSIGID()));

	} // end else

      } // end else (other request i.e. menu request)

    } // end if
    // else getting the SIG already caused problems

    BaseJSPData basedat = new BaseJSPData(page_title,"sigadmin?" + request.getQueryString(),content);
    basedat.transfer(getServletContext(),rdat);

  } // end doGet

  public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    UserContext user = getUserContext(request);
    RenderData rdat = createRenderData(request,response);
    String page_title = null;
    ContentRender content = null;

    SIGContext sig = null;
    try
    { // first get the SIG context we're working with
      int sigid = Integer.parseInt(request.getParameter("sig"));
      sig = user.getSIGContext(sigid);
      changeMenuSIG(request,sig);
      
    } // end try
    catch (NumberFormatException nfe)
    { // an improperly formatted SIGID brought us down!
      page_title = "Input Error";
      logger.error("Somebody fed 'sig=" + request.getParameter("sig") + "' to this page, that's bogus");
      content = new ErrorBox(page_title,"SIG ID not valid: " + request.getParameter("sig"),"top");

    } // end catch
    catch (DataException de)
    { // unable to pull SIG data out of the database
      page_title = "Database Error";
      logger.error("database error looking up SIGID " + request.getParameter("sig"));
      content = new ErrorBox(page_title,"Database error accessing SIG: " + de.getMessage(),"top");

    } // end catch

    if (logger.isDebugEnabled() && (sig!=null))
      logger.debug("SIGAdmin/doPost operating on SIG \"" + sig.getName() + "\" ("
		   + String.valueOf(sig.getSIGID()) + ")");

    if (content==null)
    { // now decide what to do based on the "cmd" parameter
      String cmd = request.getParameter("cmd");
      if (cmd==null)
	cmd = "???";  // something pretty much guaranteed not to be used

      if (logger.isDebugEnabled())
	logger.debug("SIGAdmin/doPost command value = " + cmd);

      if (cmd.equals("P"))
      { // we just finished editing the profile...
	if (sig.canModifyProfile())
	{ // construct the edit profile dialog and load it up for use
	  EditSIGProfileDialog dlg = makeEditSIGProfileDialog();
	  dlg.setupDialogBasic(getVeniceEngine(),sig);

	  if (dlg.isButtonClicked(request,"cancel"))
	  { // go back to our desired location
	    String target = "sigadmin?sig=" + String.valueOf(sig.getSIGID());
	    String url = response.encodeRedirectURL(rdat.getFullServletPath(target));
	    response.sendRedirect(url);
	    return;

	  } // end if ("cancel" button clicked)

	  if (dlg.isButtonClicked(request,"update"))
	  { // begin updating the SIG's contents
	    dlg.loadValues(request);  // do value loading now

	    try
	    { // attempt to change the SIG profile now...
	      dlg.doDialog(sig);

	      // go back to the Admin menu on success
	      clearMenu(request);  // menu title may have been invalidated
	      String target = "sigadmin?sig=" + String.valueOf(sig.getSIGID());
	      String url = response.encodeRedirectURL(rdat.getFullServletPath(target));
	      response.sendRedirect(url);
	      return;

	    } // end try
	    catch (ValidationException ve)
	    { // simple validation exception
	      if (logger.isDebugEnabled())
		logger.debug("validation failure: " + ve.getMessage());
	      dlg.resetOnError(sig,ve.getMessage() + " Please try again.");
	      page_title = dlg.getTitle();
	      content = dlg;

	    } // end catch
	    catch (DataException de)
	    { // database error updating the SIG values
	      page_title = "Database Error";
	      logger.error("DB error updating SIG: " + de.getMessage(),de);
	      content = new ErrorBox(page_title,"Database error updating SIG: " + de.getMessage(),
				     "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	    } // end catch
	    catch (AccessError ae)
	    { // access error changing the SIG values
	      page_title = "Access Error";
	      logger.error("Access error updating SIG: " + ae.getMessage(),ae);
	      content = new ErrorBox(page_title,ae.getMessage(),
				     "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	    } // end catch

	  } // end if ("update" button pressed);
	  else
	  { // what the hell was that button?
	    page_title = "Internal Error";
	    logger.error("no known button click on SIGAdmin.doPost, cmd=P");
	    content = new ErrorBox(page_title,"Unknown command button pressed",
				   "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	  } // end else

	} // end if
	else
	{ // no access - sorry, dude
	  page_title = "Unauthorized";
	  logger.error("tried to edit SIG profile without access...naughty naughty!");
	  content = new ErrorBox(page_title,"You do not have access to modify this SIG's profile.",
				 "sigadmin?sig=" + String.valueOf(sig.getSIGID()));

	} // end else

      } // end if (editing profile)
      else // unknown command
      { // just use that to redirect to the GET url
	String target = "sigadmin?sig=" + String.valueOf(sig.getSIGID()) + "&cmd=" + cmd;
	String url = response.encodeRedirectURL(rdat.getFullServletPath(target));
	response.sendRedirect(url);
	return;
	
      } // end else (unknown command)

    } // end if
    // else getting the SIG already caused problems

    BaseJSPData basedat = new BaseJSPData(page_title,"sigadmin?sig=" + String.valueOf(sig.getSIGID()),content);
    basedat.transfer(getServletContext(),rdat);

  } // end doPost

} // end class SIGAdmin
