/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Community System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.*;
import com.silverwrist.venice.ValidationException;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.servlets.format.*;

public class SIGOperations extends VeniceServlet
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static final String UNJOIN_CONFIRM_ATTR = "servlets.SIGOperations.unjoin.confirm";
  private static final String UNJOIN_CONFIRM_PARAM = "confirm";

  private static Category logger = Category.getInstance(SIGOperations.class.getName());

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private static SIGContext getSIGParameter(ServletRequest request, UserContext user)
      throws ValidationException, DataException
  {
    String str = request.getParameter("sig");
    if (str==null)
    { // no SIG parameter - bail out now!
      logger.error("SIG parameter not specified!");
      throw new ValidationException("No SIG specified.");

    } // end if

    try
    { // turn the string into a SIGID, and thence to a SIGContext
      int sigid = Integer.parseInt(str);
      return user.getSIGContext(sigid);

    } // end try
    catch (NumberFormatException nfe)
    { // error in Integer.parseInt
      logger.error("Cannot convert SIG parameter '" + str + "'!");
      throw new ValidationException("Invalid SIG parameter.");

    } // end catch

  } // end getSIGParameter

  private JoinKeyDialog makeJoinKeyDialog()
  {
    final String desired_name = "JoinKeyDialog";
    DialogCache cache = DialogCache.getDialogCache(getServletContext());

    if (!(cache.isCached(desired_name)))
    { // create a template and save it off
      JoinKeyDialog template = new JoinKeyDialog();
      cache.saveTemplate(template);

    } // end if

    // return a new copy
    return (JoinKeyDialog)(cache.getNewDialog(desired_name));

  } // end makeJoinKeyDialog

  private CreateSIGDialog makeCreateSIGDialog() throws ServletException
  {
    final String desired_name = "CreateSIGDialog";
    DialogCache cache = DialogCache.getDialogCache(getServletContext());

    if (!(cache.isCached(desired_name)))
    { // create a template and save it off
      CreateSIGDialog template = new CreateSIGDialog(getCountryList(),getLanguageList());
      cache.saveTemplate(template);

    } // end if

    // return a new copy
    return (CreateSIGDialog)(cache.getNewDialog(desired_name));

  } // end makeCreateSIGDialog

  /*--------------------------------------------------------------------------------
   * Overrides from class HttpServlet
   *--------------------------------------------------------------------------------
   */

  public String getServletInfo()
  {
    String rc = "SIGOperations servlet - General SIG operations (join, unjoin, etc.)\n"
              + "Part of the Venice Web Communities System\n";
    return rc;

  } // end getServletInfo

  public void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    UserContext user = getUserContext(request);
    RenderData rdat = createRenderData(request,response);
    String page_title = null;
    Object content = null;

    // get the command we want to use
    String cmd = request.getParameter("cmd");
    if (cmd==null)
      cmd = "???";

    if (cmd.equals("J"))
    { // "J" = "Join" (requires SIG parameter)
      try
      { // get the SIG parameter first!
	SIGContext sig = getSIGParameter(request,user);

	if (sig.canJoin())
	{ // OK, we can join the SIG, now, is it public or private?
	  if (sig.isPublicSIG())
	  { // attempt to join right now!  (no join key required)
	    sig.join(null);

	    // success! display the "welcome" page
	    content = new SIGWelcome(sig);
	    page_title = "Welcome to " + sig.getName();
	    clearMenu(request);  // force the clear to regen the menus
	    changeMenuSIG(request,sig);

	  } // end if (public SIG)
	  else
	  { // we need to prompt them for the join key...
	    JoinKeyDialog dlg = makeJoinKeyDialog();
	    dlg.setupDialog(sig);
	    content = dlg;
	    page_title = "Join SIG";
	    changeMenuSIG(request,sig);

	  } // end else (private SIG)

	} // end if (allowed to join the SIG)
	else
	{ // throw an error
	  page_title = "Error";
	  content = new ErrorBox("SIG Error","You are not permitted to join this SIG.","top");

	} // end else

      } // end try
      catch (AccessError ae)
      { // access error
	page_title = "Access Error";
	content = new ErrorBox(page_title,"Unable to join SIG: " + ae.getMessage(),"top");

      } // end catch
      catch (DataException de)
      { // data exception doing something
	page_title = "Database Error";
	content = new ErrorBox(page_title,"Database error joining SIG: " + de.getMessage(),"top");

      } // end catch
      catch (ValidationException ve)
      { // validation error - wrong parameters, likely
	page_title = "Error";
	content = new ErrorBox(null,ve.getMessage(),"top");

      } // end catch

    } // end if ("J" command)
    else if (cmd.equals("U"))
    { // "U" = "Unjoin (requires SIG parameter)
      try
      { // get the SIG parameter first!
	SIGContext sig = getSIGParameter(request,user);

	if (sig.canUnjoin())
	{ // OK, let's test for a confirmation...
	  if (ConfirmBox.isConfirmed(request,UNJOIN_CONFIRM_ATTR,UNJOIN_CONFIRM_PARAM))
	  { // OK, if you say so, let's unjoin!
	    sig.unjoin();

	    // after which, let's just go back to top
	    clearMenu(request);  // force the clear to regen the menus
	    rdat.redirectTo("top");
	    return;

	  } // end if
	  else
	  { // not a proper confirmation - better display one
	    String message = "Are you sure you want to unjoin the '" + sig.getName() + "' SIG?";
	    String confirm_url = "sigops?cmd=U&sig=" + String.valueOf(sig.getSIGID());
	    String deny_url = "sig/" + sig.getAlias();
	    page_title = "Unjoining SIG";
	    content = new ConfirmBox(request,UNJOIN_CONFIRM_ATTR,UNJOIN_CONFIRM_PARAM,page_title,
				     message,confirm_url,deny_url);

	  } // end else

	} // end if
	else
	{ // throw an error
	  page_title = "Error";
	  content = new ErrorBox("SIG Error","You cannot unjoin this SIG.","top");

	} // end else

      } // end try
      catch (AccessError ae)
      { // access error
	page_title = "Access Error";
	content = new ErrorBox(page_title,"Unable to unjoin SIG: " + ae.getMessage(),"top");

      } // end catch
      catch (DataException de)
      { // data exception doing something
	page_title = "Database Error";
	content = new ErrorBox(page_title,"Database error unjoining SIG: " + de.getMessage(),"top");

      } // end catch
      catch (ValidationException ve)
      { // validation error - wrong parameters, likely
	page_title = "Error";
	content = new ErrorBox(null,ve.getMessage(),"top");

      } // end catch

    } // end else if ("U" command)
    else if (cmd.equals("C"))
    { // "C" - Create SIG (no parameters)
      if (user.canCreateSIG())
      { // present the "Create New SIG" dialog
	CreateSIGDialog dlg = makeCreateSIGDialog();
	dlg.setupDialog(getVeniceEngine());

	// prepare for display
	page_title = dlg.getTitle();
	content = dlg;
	changeMenuTop(request);

      } // end if
      else
      { // the user isn't permitted to make new SIGs...
	page_title = "Error";
	content = new ErrorBox("SIG Error","You are not permitted to create SIGs.","top");

      } // end else

    } // end else if ("C" command)
    else
    { // this is an error!
      page_title = "Internal Error";
      logger.error("invalid command to SIGOperations.doGet: " + cmd);
      content = new ErrorBox(page_title,"Invalid command to SIGOperations.doGet","top");

    } // end else

    BaseJSPData basedat = new BaseJSPData(page_title,"sigops?" + request.getQueryString(),content);
    basedat.transfer(getServletContext(),rdat);

  } // end doGet

  public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    UserContext user = getUserContext(request);
    RenderData rdat = createRenderData(request,response);
    String page_title = null;
    Object content = null;

    // get the command we want to use
    String cmd = request.getParameter("cmd");
    if (cmd==null)
      cmd = "???";

    if (cmd.equals("J"))
    { // "J" = Join SIG (requires SIG parameter)
      try
      { // get the SIG we're talking about
	SIGContext sig = getSIGParameter(request,user);
	JoinKeyDialog dlg = makeJoinKeyDialog();

	if (dlg.isButtonClicked(request,"cancel"))
	{ // canceled join - return to SIG opening page
	  rdat.redirectTo("sig/" + sig.getAlias());
	  return;

	} // end if

	if (dlg.isButtonClicked(request,"join"))
	{ // they clicked the "join" button
	  if (sig.canJoin())
	  { // we are permitted to join this SIG...
	    dlg.loadValues(request);  // load the dialog
	    
	    try
	    { // attempt to join the SIG!
	      dlg.doDialog(sig);

	      // success! display the "welcome" page
	      content = new SIGWelcome(sig);
	      page_title = "Welcome to " + sig.getName();
	      clearMenu(request);  // force the clear to regen the menus
	      changeMenuSIG(request,sig);

	    } // end try
	    catch (ValidationException ve2)
	    { // here, a validation exception causes us to recycle and retry
	      dlg.resetOnError(ve2.getMessage() + " Please try again.");
	      content = dlg;
	      page_title = "Join SIG";
	      changeMenuSIG(request,sig);
	    
	    } // end catch
	    catch (AccessError ae)
	    { // this is probably a bogus key - let them retry
	      dlg.resetOnError(ae.getMessage() + " Please try again.");
	      content = dlg;
	      page_title = "Join SIG";
	      changeMenuSIG(request,sig);
	    
	    } // end catch

	  } // end if
	  else
	  { // throw an error
	    page_title = "Error";
	    content = new ErrorBox("SIG Error","You are not permitted to join this SIG.","top");

	  } // end else

	} // end if ("join" button clicked)
	else
	{ // error - don't know what button was clicked
	  page_title = "Internal Error";
	  logger.error("no known button click on SIGOperations.doPost, cmd=J");
	  content = new ErrorBox(page_title,"Unknown command button pressed","top");

	} // end else

      } // end try
      catch (DataException de)
      { // database error joining something
	page_title = "Database Error";
	content = new ErrorBox(page_title,"Database error joining SIG: " + de.getMessage(),"top");

      } // end catch
      catch (ValidationException ve)
      { // validation error - bogus parameter, I think
	page_title = "Error";
	content = new ErrorBox(null,ve.getMessage(),"top");

      } // end catch

    } // end if ("J" command)
    else if (cmd.equals("C"))
    { // "C" = Create New SIG
      if (user.canCreateSIG())
      { // load the "Create SIG" dialog
	CreateSIGDialog dlg = makeCreateSIGDialog();
	dlg.setupDialog(getVeniceEngine());

	if (dlg.isButtonClicked(request,"cancel"))
	{ // canceled create - return to top page
	  rdat.redirectTo("top");
	  return;

	} // end if

	if (dlg.isButtonClicked(request,"create"))
	{ // OK, they actually want to create the new SIG...
	  dlg.loadValues(request);  // load the form data

	  try
	  { // attempt to create the SIG!
	    SIGContext sig = dlg.doDialog(user);

	    // created successfully - display a "new SIG welcome" page
	    content = new NewSIGWelcome(sig);
	    page_title = "SIG Created";
	    changeMenuSIG(request,sig);  // display menus for the first time!

	  } // end try
	  catch (ValidationException ve2)
	  { // here, a validation exception causes us to recycle and retry
	    dlg.resetOnError(ve2.getMessage() + " Please try again.");
	    content = dlg;
	    page_title = dlg.getTitle();
	    changeMenuTop(request);
	    
	  } // end catch
	  catch (AccessError ae)
	  { // this is probably a bogus key - let them retry
	    dlg.resetOnError(ae.getMessage() + " Please try again.");
	    content = dlg;
	    page_title = dlg.getTitle();
	    changeMenuTop(request);
	    
	  } // end catch
	  catch (DataException de)
	  { // database error doing something
	    page_title = "Database Error";
	    content = new ErrorBox(page_title,"Database error creating SIG: " + de.getMessage(),"top");

	  } // end catch

	} // end if
	else
	{ // error - don't know what button was clicked
	  page_title = "Internal Error";
	  logger.error("no known button click on SIGOperations.doPost, cmd=C");
	  content = new ErrorBox(page_title,"Unknown command button pressed","top");

	} // end else

      } // end if
      else
      { // the user isn't permitted to make new SIGs...
	page_title = "Error";
	content = new ErrorBox("SIG Error","You are not permitted to create SIGs.","top");

      } // end else

    } // end else if ("C" command)
    else
    { // this is an error!
      page_title = "Internal Error";
      logger.error("invalid command to SIGOperations.doPost: " + cmd);
      content = new ErrorBox(page_title,"Invalid command to SIGOperations.doPost","top");

    } // end else

    BaseJSPData basedat = new BaseJSPData(page_title,"top",content);
    basedat.transfer(getServletContext(),rdat);

  } // end doPost

} // end class SIGOperations
