/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Community System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.*;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.servlets.format.*;

public class Variables
{
  /*--------------------------------------------------------------------------------
   * Static data values
   *--------------------------------------------------------------------------------
   */

  // ServletContext ("application") attributes
  protected static final String ENGINE_ATTRIBUTE = "com.silverwrist.venice.core.Engine";
  protected static final String COUNTRYLIST_ATTRIBUTE = "com.silverwrist.venice.db.CountryList";
  protected static final String LANGUAGELIST_ATTRIBUTE = "com.silverwrist.venice.db.LanguageList";
  protected static final String TOPMENU_ATTRIBUTE = "com.silverwrist.venice.servlets.MenuTop";

  // HttpSession ("session") attributes
  protected static final String USERCTXT_ATTRIBUTE = "user.context";
  protected static final String MENU_ATTRIBUTE = "current.menu";

  // Servlet initialization parameters
  protected static final String ENGINE_INIT_PARAM = "venice.config";

  // Cookie name
  public static final String LOGIN_COOKIE = "VeniceLogin";

  private static Category logger = Category.getInstance(Variables.class.getName());
  private static Integer engine_gate = new Integer(0);

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  public static VeniceEngine getVeniceEngine(ServletContext ctxt) throws ServletException
  {
    synchronized(engine_gate)
    { // we must synchronize efforts to access the engine
      Object foo = ctxt.getAttribute(ENGINE_ATTRIBUTE);
      if (foo!=null)
	return (VeniceEngine)foo;

      String cfgfile = ctxt.getInitParameter(ENGINE_INIT_PARAM);  // get the config file name
      logger.info("Initializing Venice engine using config file: " + cfgfile);

      try
      { // extract the configuration file name and create the engine
	VeniceEngine engine = Startup.createEngine(cfgfile);
	ctxt.setAttribute(ENGINE_ATTRIBUTE,engine);
	return engine;

      } // end try
      catch (ConfigException e)
      { // configuration failed!  post an error message
	logger.fatal("Engine configuration failed: " + e.getMessage(),e);
	throw new ServletException("Venice engine configuration failed: " + e.getMessage(),e);

      } // end catch
      catch (DataException e2)
      { // configuration failed!  post an error message
	logger.fatal("Engine data load failed: " + e2.getMessage(),e2);
	throw new ServletException("Venice engine data load failed: " + e2.getMessage(),e2);

      } // end catch

    } // end synchronized block

  } // end getVeniceEngine

  public static UserContext getUserContext(ServletContext ctxt, ServletRequest request, HttpSession session)
      throws ServletException
  {
    Object uctmp = session.getAttribute(USERCTXT_ATTRIBUTE);
    if (uctmp!=null)
      return (UserContext)uctmp;

    try
    { // use the Venice engine to create a new user context and save it off
      VeniceEngine engine = getVeniceEngine(ctxt);
      UserContext user = engine.createUserContext(request.getRemoteAddr());
      session.setAttribute(USERCTXT_ATTRIBUTE,user);
      return user;

    } // end try
    catch (DataException e)
    { // context creation failed - remap the exception
      logger.error("Session creation failed: " + e.getMessage(),e);
      throw new ServletException("Session creation failed: " + e.getMessage(),e);

    } // end catch

  } // end getUserContext

  public static void putUserContext(HttpSession session, UserContext user)
  {
    session.setAttribute(USERCTXT_ATTRIBUTE,user);
    session.removeAttribute(MENU_ATTRIBUTE);
    
  } // end putUserContext

  public static void clearUserContext(HttpSession session)
  {
    session.removeAttribute(USERCTXT_ATTRIBUTE);
    session.removeAttribute(MENU_ATTRIBUTE);

  } // end clearUserContext

  public static List getCountryList(ServletContext ctxt) throws ServletException
  {
    Object foo = ctxt.getAttribute(COUNTRYLIST_ATTRIBUTE);
    if (foo!=null)
      return (List)foo;

    VeniceEngine engine = getVeniceEngine(ctxt);

    try
    { // get the country list via the engine and save it
      List rc = engine.getCountryList();
      ctxt.setAttribute(COUNTRYLIST_ATTRIBUTE,rc);
      return rc;

    } // end try
    catch (DataException e)
    { // the country list could not be retrieved
      logger.error("Failed to retrieve country list from engine: " + e.getMessage(),e);
      throw new ServletException("Country list retrieval failed: " + e.getMessage(),e);

    } // end catch

  } // end getCountryList

  public static List getLanguageList(ServletContext ctxt) throws ServletException
  {
    Object foo = ctxt.getAttribute(LANGUAGELIST_ATTRIBUTE);
    if (foo!=null)
      return (List)foo;

    VeniceEngine engine = getVeniceEngine(ctxt);

    try
    { // get the country list via the engine and save it
      List rc = engine.getLanguageList();
      ctxt.setAttribute(LANGUAGELIST_ATTRIBUTE,rc);
      return rc;

    } // end try
    catch (DataException e)
    { // the country list could not be retrieved
      logger.error("Failed to retrieve language list from engine: " + e.getMessage(),e);
      throw new ServletException("Language list retrieval failed: " + e.getMessage(),e);

    } // end catch

  } // end getLanguageList

  public static ContentRender getMenu(HttpSession session)
  {
    return (ContentRender)(session.getAttribute(MENU_ATTRIBUTE));

  } // end getMenu

  public static void setMenuTop(ServletContext ctxt, HttpSession session)
  {
    Object obj = session.getAttribute(MENU_ATTRIBUTE);
    if ((obj==null) || !(obj instanceof MenuTop))
    { // look for the common MenuTop object and set it to the session context
      obj = ctxt.getAttribute(TOPMENU_ATTRIBUTE);
      if (obj==null)
      { // we don't have a common MenuTop yet...make one
	MenuTop mt = new MenuTop();
	ctxt.setAttribute(TOPMENU_ATTRIBUTE,mt);
	obj = mt;

      } // end if

      session.setAttribute(MENU_ATTRIBUTE,obj);

    } // end if

  } // end setMenuTop

  public static void setMenuSIG(HttpSession session, SIGContext sig)
  {
    Object obj = session.getAttribute(MENU_ATTRIBUTE);
    boolean do_change;
    if ((obj==null) || !(obj instanceof MenuSIG))
      do_change = true;
    else
    { // look at the actual SIGIDs underlying the MenuSIG
      MenuSIG tmp = (MenuSIG)obj;
      do_change = (tmp.getID()!=sig.getSIGID());

    } // end else

    if (do_change)
    { // switch to the appropriate MenuSIG
      MenuSIG ms = new MenuSIG(sig);
      session.setAttribute(MENU_ATTRIBUTE,ms);
      
    } // end if

  } // end setMenuSIG

  public static void clearMenu(HttpSession session)
  {
    session.removeAttribute(MENU_ATTRIBUTE);

  } // end clearMenu

  public static void failIfNull(Object obj) throws ServletException
  {
    if (obj==null)
      throw new ServletException("improper context - data not set");

  } // end failIfNull

} // end class Variables
